"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_crypto_1 = require("node:crypto");
const microservice_1 = require("@jfrog-pipelines/microservice");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const logger_1 = require("@jfrog-pipelines/logger");
const SendApprovalRequest_1 = require("../SendApprovalRequest");
const modules_1 = require("../../modules");
jest.mock('@jfrog-pipelines/configuration', () => {
    return {
        ConfigFactory: { config: { apiToken: 'apiToken' } }
    };
});
jest.mock('@jfrog-pipelines/logger', () => {
    const actualTracker = jest.requireActual('@jfrog-pipelines/logger');
    return {
        ...actualTracker,
        LoggerFactory: {
            getLogger: jest.fn().mockReturnValue({
                info: jest.fn(),
                error: jest.fn(),
                debug: jest.fn(),
                trace: jest.fn(),
                warn: jest.fn()
            }),
            logEvent: jest.fn()
        }
    };
});
const lockResponse = { id: (0, node_crypto_1.randomUUID)() };
const logger = logger_1.LoggerFactory.getLogger();
describe('SendApprovalRequest', () => {
    microservice_1.applicationContext.set(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER, { systemServiceInstancesId: 1 });
    const sendRequestApprovalsStatus = new SendApprovalRequest_1.SendApprovalRequests('session');
    it('should get approve one step and one cannot execute', async () => {
        const eventLog = {
            timestamp: expect.any(String),
            domain: 'step-service',
            eventType: 'approvalrequests',
            status: 'success',
            noOfRequests: 1,
            projectId: 1,
            pipelinesSessionId: 'session'
        };
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest
            .spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility')
            .mockResolvedValueOnce({ canExecuteStep: true });
        jest
            .spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility')
            .mockResolvedValueOnce({ canExecuteStep: false });
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'requestStepApproval').mockResolvedValueOnce({});
        jest.spyOn(modules_1.WaitingGroup.prototype, 'getApprovalGatesStepIds').mockReturnValueOnce([1, 2]);
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockResolvedValue(lockResponse);
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockImplementationOnce(jest.fn());
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(internal_clients_1.PipelinesClient.prototype.getStepExecutionEligibility).toHaveBeenCalledTimes(2);
        expect(internal_clients_1.PipelinesClient.prototype.getStepExecutionEligibility).toHaveBeenNthCalledWith(1, 1);
        expect(internal_clients_1.PipelinesClient.prototype.getStepExecutionEligibility).toHaveBeenNthCalledWith(2, 2);
        expect(internal_clients_1.PipelinesClient.prototype.requestStepApproval).toHaveBeenCalledTimes(1);
        expect(internal_clients_1.PipelinesClient.prototype.requestStepApproval).toHaveBeenCalledWith(1);
        expect(obtainLockSpy).toHaveBeenCalledTimes(1);
        expect(releaseLockSpy).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledWith(eventLog);
    });
    it('should log warning when error occurred when trying to obtain lock', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockRejectedValueOnce('error');
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockImplementationOnce(jest.fn());
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.warn).toHaveBeenCalledTimes(1);
        expect(obtainLockSpy).toHaveBeenCalledTimes(1);
        expect(releaseLockSpy).toHaveBeenCalledTimes(0);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(0);
    });
    it('should log warning when obtain lock returned null', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockResolvedValueOnce(undefined);
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockImplementationOnce(jest.fn());
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.warn).toHaveBeenCalledTimes(1);
        expect(obtainLockSpy).toHaveBeenCalledTimes(1);
        expect(releaseLockSpy).toHaveBeenCalledTimes(0);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(0);
    });
    it('should log warning when unable to check step execution eligibility', async () => {
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility').mockRejectedValueOnce('error');
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockResolvedValue(lockResponse);
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockImplementationOnce(jest.fn());
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(logger.warn).toHaveBeenCalledTimes(3);
        expect(obtainLockSpy).toHaveBeenCalledTimes(1);
        expect(releaseLockSpy).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(0);
    });
    it('should log warning when unable to approve step and then log event', async () => {
        const eventLog = {
            timestamp: expect.any(String),
            domain: 'step-service',
            eventType: 'approvalrequests',
            status: 'failed',
            noOfRequests: 1,
            projectId: 1,
            pipelinesSessionId: 'session'
        };
        const waitingGroupMock = [
            {
                projectId: 1,
                requiresApproval: true,
                rejectionContext: {},
                getApprovalGatesStepIds: () => {
                    return [1, 2];
                }
            }
        ];
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'getStepExecutionEligibility').mockResolvedValue({ canExecuteStep: true });
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'requestStepApproval').mockRejectedValueOnce('errorApproval');
        jest.spyOn(internal_clients_1.PipelinesClient.prototype, 'requestStepApproval').mockResolvedValueOnce({});
        jest.spyOn(modules_1.WaitingGroup.prototype, 'getApprovalGatesStepIds').mockReturnValueOnce([1, 2]);
        const obtainLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'obtainLock').mockResolvedValue(lockResponse);
        const releaseLockSpy = jest.spyOn(microservice_1.Lock.prototype, 'releaseLock').mockImplementationOnce(jest.fn());
        await sendRequestApprovalsStatus.getWaitingAndRejectedGroupsAfterApprovalProcessing(waitingGroupMock);
        expect(internal_clients_1.PipelinesClient.prototype.requestStepApproval).toHaveBeenCalledTimes(2);
        expect(internal_clients_1.PipelinesClient.prototype.requestStepApproval).toHaveBeenNthCalledWith(1, 1);
        expect(internal_clients_1.PipelinesClient.prototype.requestStepApproval).toHaveBeenNthCalledWith(2, 2);
        expect(logger.warn).toHaveBeenCalledTimes(1);
        expect(obtainLockSpy).toHaveBeenCalledTimes(1);
        expect(releaseLockSpy).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledTimes(1);
        expect(logger_1.LoggerFactory.logEvent).toHaveBeenCalledWith(eventLog);
    });
});
//# sourceMappingURL=SendApprovalRequest.test.js.map