"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueueStepsToJobNodePool = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const acl_1 = require("../acl");
const EXECUTION_TYPE_JOB = 'job';
const KUBERNETES_INTEGRATION = 'kubernetesConfig';
const EXECUTION_TYPE_TAG = 'executionType';
class QueueStepsToJobNodePool {
    constructor() {
        this.logger = logger_1.LoggerFactory.getLogger('QueueStepsToJobNodePool');
    }
    async checkEligibilityForQueuingToJobNodePool(nodePool) {
        try {
            if (!nodePool.isOnDemand) {
                return false;
            }
            const projectIntegrations = await this.getProjectIntegration(nodePool.projectIntegrationId);
            const isKubernetesNodePool = this.checkIfNodePoolIsKubernetes(projectIntegrations);
            const isExecutionTypeJob = this.checkIfExecutionTypeJob(nodePool);
            return isKubernetesNodePool && isExecutionTypeJob;
        }
        catch (error) {
            throw new Error(`Cannot extract projectIntegrations for projectIntegrationId: ${nodePool.projectIntegrationId}. Getting error: ${(0, node_util_1.inspect)(error)}`);
        }
    }
    executeStepsUsingJobNodePool(waitingGroup, stepletsByStepId) {
        const execuatableStepsResponse = this.getExecutableSteps(waitingGroup);
        this.queueToJobNodePool(execuatableStepsResponse, stepletsByStepId);
    }
    async getProjectIntegration(projectId) {
        const integrationRepository = new acl_1.IntegrationRepository();
        return integrationRepository.getProjectIntegrationById(projectId);
    }
    checkIfNodePoolIsKubernetes(projectIntegrations) {
        if (projectIntegrations.masterIntegrationName === KUBERNETES_INTEGRATION) {
            return true;
        }
        return false;
    }
    checkIfExecutionTypeJob(nodePool) {
        if (!nodePool.providerMetadataPropertyBag.customTags) {
            return false;
        }
        const customTags = nodePool.providerMetadataPropertyBag.customTags;
        for (const tag of customTags) {
            if (this.isJobExecutionTypeTag(tag)) {
                return true;
            }
        }
        return false;
    }
    isJobExecutionTypeTag(tag) {
        if (tag.Key === EXECUTION_TYPE_TAG && tag.Value.toLowerCase() === EXECUTION_TYPE_JOB) {
            return true;
        }
        return false;
    }
    getExecutableSteps(waitingGroup) {
        waitingGroup.isJobNodePool = true;
        const executableSteps = waitingGroup.getStepsToExecute();
        const executableStepIds = (0, utility_1.map)(executableSteps, 'stepId');
        return { executableSteps: executableSteps, executableStepIds: executableStepIds };
    }
    queueToJobNodePool(execuatableStepsResponse, executableStepletsByStepId) {
        const messagePayloads = [];
        const steps = execuatableStepsResponse.executableSteps;
        for (const step of steps) {
            const stepId = step.stepId;
            for (const steplet of executableStepletsByStepId[stepId]) {
                const messagePayload = this.getMessagePayload(stepId, step, steplet);
                messagePayloads.push(messagePayload);
            }
        }
        this.logger.warn((0, node_util_1.inspect)(messagePayloads));
    }
    getMessagePayload(stepId, step, steplet) {
        return {
            stepId: stepId,
            step: step,
            steplet: steplet,
            image: steplet.configPropertyBag.imageName,
            tag: steplet.configPropertyBag.imageTag
        };
    }
}
exports.QueueStepsToJobNodePool = QueueStepsToJobNodePool;
//# sourceMappingURL=queueStepsToJobNodePool.js.map