'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var self = queueStepsToNodePools;
module.exports = self;
var _ = require('underscore');
var async = require('async');
var uuid = require('node-uuid');
var { CALL_OBJECT_LIMIT_FOR_BATCH, CALL_OBJECT_LIMIT, WILDCARD_AUDIENCE } = require('../constants.js');
var LAST_JOB_QUEUED_INTERVAL_IN_MINS = 2;
var LOCK_TIMEOUT_IN_MS = 1 * 60 * 1000;
var LOCK_TIMER_INTERVAL_IN_MS = 50 * 1000;
var { getElapsedTimeSinceStartTime } = require('./utils');
var BatchStepletConsoleAdapter = require('../BatchStepletConsoleAdapter.js');
var nodeActions = require('../nodeActions.js');
var getDownstreamStepIds = require('./getDownStreamStepIds.js');
var sortStepsByDependency = require('./sortStepsByDependency.js');
var checkGroupExecutionEligibility = require('./checkGroupExecutionEligibility.js');
const { DataAdapterFactory } = require('../modules/dataAdapter/DataAdapterFactory.js');
const logger_1 = require("@jfrog-pipelines/logger");
const acl_1 = require("../acl");
const waitingStepsByRunAdjacency_1 = require("../waitingStepsByRunAdjacency");
const waitingGroupManager_1 = require("../waitingGroupManager");
const microservice_1 = require("@jfrog-pipelines/microservice");
const StepScheduler_1 = require("../StepScheduler");
const utility_1 = require("@jfrog-pipelines/utility");
function queueStepsToNodePools(externalBag, callback) {
    var who = externalBag.who + '|' + queueStepsToNodePools.name;
    logger.debug(who, 'Inside');
    let startTime = Date.now();
    var bag = {
        who: who,
        suAdapter: externalBag.suAdapter,
        statusCodes: global.systemCodesByGroup.status,
        nodePools: externalBag.nodePools,
        waitingGroups: externalBag.waitingGroups,
        processingGroups: externalBag.processingGroups,
        allowedNodePoolsByProject: externalBag.allowedNodePoolsByProject,
        jFrogPipelinesSessionId: externalBag.jFrogPipelinesSessionId,
        defaultPriority: externalBag.defaultPriority,
        rejectedGroups: externalBag.rejectedGroups,
        result: {
            waitingGroups: [],
            processingGroups: []
        },
        successfulNodePools: 0,
        failedNodePools: 0
    };
    async.series([
        _checkInputParams.bind(null, bag),
        _processNodePools.bind(null, bag),
        _setResult.bind(null, bag)
    ], function (err) {
        let nodePoolQueueStatus = "success";
        if (err)
            nodePoolQueueStatus = "failed";
        var stopTime = Date.now();
        var timeDiff = new Date(stopTime - startTime);
        var finalTime = util.format("durationMins=%s - %s minutes, %s seconds, and " +
            "%s milliseconds", (timeDiff.getTime() / 1000 / 60), timeDiff.getMinutes(), timeDiff.getSeconds(), timeDiff.getMilliseconds());
        logger.debug(bag.who, "Finished queueStepsToNodePools -", finalTime);
        const eventData = {
            timestamp: new Date(stopTime).toISOString(),
            domain: global.msName,
            eventType: "allnodepoolsprocess",
            status: nodePoolQueueStatus,
            durationMillis: timeDiff.valueOf(),
            successfulNodePools: bag.successfulNodePools,
            failedNodePools: bag.failedNodePools,
            pipelinesSessionId: bag.jFrogPipelinesSessionId
        };
        logger_1.LoggerFactory.logEvent(eventData);
        return callback(err, bag.result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.debug(who, 'Inside');
    if (!bag.suAdapter)
        return next(new Error(':suAdapter is missing'));
    bag.waitingGroups = _.reject(bag.waitingGroups, (waitingGroup) => {
        return waitingGroup.requiresApproval === true;
    });
    return next();
}
function _processNodePools(bag, next) {
    if (_.isEmpty(bag.nodePools))
        return next();
    var who = bag.who + '|' + _processNodePools.name;
    logger.debug(who, 'Inside');
    var processingGroupsMap = {};
    _.each(bag.processingGroups, function (processingGroup) {
        let nodePool = _.find(bag.allowedNodePoolsByProject[processingGroup.projectId], function (nodePool) {
            return nodePool.name === processingGroup.nodePoolName;
        });
        if (nodePool) {
            if (!processingGroupsMap[nodePool.id])
                processingGroupsMap[nodePool.id] = [];
            processingGroupsMap[nodePool.id].push(processingGroup);
        }
    });
    var waitingGroupsMap = {};
    _.each(bag.waitingGroups, function (waitingGroup) {
        let nodePool = _.find(bag.allowedNodePoolsByProject[waitingGroup.projectId], function (nodePool) {
            return nodePool.name === waitingGroup.nodePoolName;
        });
        if (nodePool) {
            if (!waitingGroupsMap[nodePool.id])
                waitingGroupsMap[nodePool.id] = [];
            waitingGroupsMap[nodePool.id].push(waitingGroup);
        }
    });
    async.eachLimit(bag.nodePools, 5, function (nodePool, nextNodePool) {
        var innerBag = {
            who: bag.who + '|nodePoolName:' + nodePool.name,
            suAdapter: bag.suAdapter,
            nodePool: nodePool,
            nodes: [],
            project: _.findWhere(bag.groupsProjects, { id: nodePool.projectId }),
            waitingGroups: waitingGroupsMap[nodePool.id] || [],
            statusCodes: bag.statusCodes,
            processingGroups: processingGroupsMap[nodePool.id] || [],
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId,
            error: false,
            defaultPriority: bag.defaultPriority,
            rejectedGroups: bag.rejectedGroups,
            stepsQueued: 0,
            groupsProjects: bag.groupsProjects
        };
        if (!nodePool.isOnDemand && _.isEmpty(innerBag.waitingGroups))
            return process.nextTick(nextNodePool);
        if (nodePool.isOrphaned)
            innerBag.nodes =
                bag.orphanedOnDemandNodesByProjectId[nodePool.projectId];
        let startTime = new Date();
        let tasks = [
            __obtainLock.bind(null, innerBag),
            __getNodePool.bind(null, innerBag),
            __getAndFilterAvailableNodes.bind(null, innerBag),
            __getBuildPlaneMessagesCount.bind(null, innerBag),
            __getWaitingStepsByRunAdjacencies.bind(null, innerBag),
            __setReadyAtByRunAdjacencies.bind(null, innerBag),
            __rejectGroupByNodesAndNonRootEnforcement.bind(null, innerBag),
            __findAvailableNodeCount.bind(null, innerBag),
            __tryAndQueueStepsNew.bind(null, innerBag),
            __releaseLock.bind(null, innerBag),
            __publishToNPM.bind(null, innerBag)
        ];
        async.series(tasks, function (err) {
            let nodePoolQueueStatus = "success";
            var stopTime = Date.now();
            var timeDiff = new Date(stopTime - startTime);
            var finalTime = util.format("durationMins=%s - %s minutes, %s seconds, and " +
                "%s milliseconds", (timeDiff.getTime() / 1000 / 60), timeDiff.getMinutes(), timeDiff.getSeconds(), timeDiff.getMilliseconds());
            logger.debug(bag.who, `Finished processing nodePool - ${nodePool.name} and it took ${finalTime}`);
            if (err || bag.error) {
                logger.debug(bag.who, 'Failed to process nodePool:', nodePool.name, 'with error:', err || bag.error);
                nodePoolQueueStatus = 'failed';
                bag.failedNodePools++;
            }
            else {
                logger.debug(bag.who, 'Successfully processed nodePool:', nodePool.name);
                bag.successfulNodePools++;
            }
            const eventData = {
                timestamp: new Date(stopTime).toISOString(),
                domain: global.msName,
                eventType: "nodepoolprocess",
                status: nodePoolQueueStatus,
                queuedStepsCount: innerBag.stepsQueued,
                durationMillis: timeDiff.valueOf(),
                nodepoolName: nodePool.name,
                pipelinesSessionId: bag.jFrogPipelinesSessionId
            };
            logger_1.LoggerFactory.logEvent(eventData);
            return nextNodePool();
        });
    }, function () {
        return next();
    });
}
async function __obtainLock(bag) {
    var who = bag.who + '|' + __obtainLock.name;
    logger.debug(who, 'Inside');
    const name = global.msName + '-nodePools-' + bag.nodePool.id;
    bag.lockService = new microservice_1.Lock(microservice_1.applicationContext.get(microservice_1.SYSTEM_SERVICE_INSTANCE_INITIALIZER)?.systemServiceInstancesId, name, LOCK_TIMEOUT_IN_MS);
    try {
        bag.lock = await bag.lockService.obtainLock();
        bag.isValidLock = true;
    }
    catch (error) {
        logger.warn('failed to obtain lock with name:', name);
        throw error;
    }
}
async function __getNodePool(bag) {
    if (!bag.isValidLock)
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __getNodePool.name;
    logger.debug(who, 'Inside');
    var query = 'nodePoolIds=' + bag.nodePool.id;
    const nodePoolRepository = new acl_1.NodePoolRepository();
    try {
        const nodePools = await nodePoolRepository.getNodePools(query);
        if (nodePools.length === 0) {
            bag.error = true;
            logger.debug(who, 'nodePools empty for query', query);
            return;
        }
        else {
            bag.nodePool = nodePools[0];
        }
    }
    catch (err) {
        logger.warn('Failed to getNodePools for query:', query, 'with error:', err);
        bag.error = true;
    }
}
async function __getAndFilterAvailableNodes(bag) {
    if (bag.error)
        return;
    if (!bag.isValidLock)
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __getAndFilterAvailableNodes.name;
    logger.debug(who, 'Inside');
    const nodePoolRepository = new acl_1.NodePoolRepository();
    try {
        bag.nodes = await nodePoolRepository.getAndFilterAvailableNodes(bag.nodePool.id, bag.statusCodes);
    }
    catch (err) {
        bag.error = true;
        logger.warn(who, util.format('Failed to getFilteredNodes for query: %s with error: %s', query, err));
    }
}
async function __getWaitingStepsByRunAdjacencies(bag) {
    if (bag.error)
        return;
    if (!bag.isValidLock)
        return;
    if (_.isEmpty(bag.waitingGroups))
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __getWaitingStepsByRunAdjacencies.name;
    logger.debug(who, 'Inside');
    try {
        bag.waitingGroups = await waitingStepsByRunAdjacency_1.WaitingStepsByRunAdjacency.getWaitingStepsByRunAdjacencies(bag.waitingGroups);
    }
    catch (err) {
        logger.warn(`Failed to get Waiting steps by Run Adjacencies with error: ${err}`);
        throw err;
    }
}
async function __setReadyAtByRunAdjacencies(bag) {
    if (bag.error)
        return;
    if (!bag.isValidLock)
        return;
    if (_.isEmpty(bag.waitingGroups))
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __setReadyAtByRunAdjacencies.name;
    logger.debug(who, 'Inside');
    const waitingGroupManager = new waitingGroupManager_1.WaitingGroupManager();
    try {
        await waitingGroupManager.setReadyAt(bag.waitingGroups);
    }
    catch (error) {
        logger.warn(error);
        throw error;
    }
}
function __setReadyAt(bag, next) {
    if (bag.error)
        return next();
    if (!bag.isValidLock)
        return next();
    if (_.isEmpty(bag.waitingGroups))
        return next();
    if (bag.nodePool.isOrphaned)
        return next();
    var who = bag.who + '|' + __setReadyAt.name;
    logger.debug(who, 'Inside');
    async.eachLimit(bag.waitingGroups, 5, function (waitingGroup, done) {
        var alreadySet = _.some(waitingGroup.groupStepIds, function (step) {
            return step.readyAt;
        });
        if (alreadySet)
            return process.nextTick(done);
        var groupStepIds = _.clone(waitingGroup.groupStepIds);
        var runStepConnections = bag.runStepConnectionsByRunId[waitingGroup.runId];
        var runStepConnectionsByStepId = _.groupBy(runStepConnections, 'stepId');
        var runStepConnectionsByOperationStepId = _.groupBy(runStepConnections, 'operationStepId');
        var runStepConnectionsByResourceName = _.groupBy(runStepConnections, 'operationRunResourceVersionName');
        async.eachSeries(waitingGroup.groupStepIds, function (stepObj, nextStepObj) {
            var _bag = {
                who: who,
                stepId: stepObj.stepId,
                runResourceVersionsById: bag.runResourceVersionsById,
                runStepConnectionsByStepId: runStepConnectionsByStepId,
                runStepConnectionsByOperationStepId: runStepConnectionsByOperationStepId,
                runStepConnectionsByResourceName: runStepConnectionsByResourceName,
            };
            getDownstreamStepIds(_bag, function (downstreamStepIds) {
                groupStepIds = _.reject(groupStepIds, function (groupStep) {
                    return _.contains(downstreamStepIds, groupStep.stepId);
                });
                return process.nextTick(nextStepObj);
            });
        }, function () {
            groupStepIds = _.sortBy(groupStepIds, function (groupStep) {
                return groupStep.priority;
            });
            var firstStep = _.first(groupStepIds);
            if (!firstStep)
                return process.nextTick(done);
            var update = {
                readyAt: new Date()
            };
            bag.suAdapter.putStepById(firstStep.stepId, update, function (err) {
                if (err) {
                    logger.debug(who, 'Failed to putStepById for id:', firstStep.stepId, 'with error:', err);
                    return done(true);
                }
                return process.nextTick(done);
            });
        });
    }, function (err) {
        if (err)
            bag.error = true;
        return next();
    });
}
function __rejectGroupByNodesAndNonRootEnforcement(bag, next) {
    if (bag.error)
        return next();
    if (!bag.isValidLock)
        return next();
    if (_.isEmpty(bag.waitingGroups))
        return next();
    if (bag.nodePool.isOrphaned)
        return next();
    var who = bag.who + '|' + __rejectGroupByNodesAndNonRootEnforcement.name;
    logger.debug(who, 'Inside');
    waitingGroupManager_1.WaitingGroupManager.rejectGroup(bag.nodePool.isOnDemand, bag.nodes, bag.waitingGroups, global.config.enforceNonRootNodes, bag.nodePool.initAsNonRoot, bag.nodePool.numberOfNodes);
    return next();
}
async function __getBuildPlaneMessagesCount(bag) {
    if (bag.error)
        return;
    if (!bag.isValidLock)
        return;
    if (_.isEmpty(bag.waitingGroups))
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __getBuildPlaneMessagesCount.name;
    logger.debug(who, 'Inside');
    const nodePoolRepository = new acl_1.NodePoolRepository();
    try {
        bag.buildPlaneMessagesCount = await nodePoolRepository.getBuildPlaneMessagesCount(bag.statusCodes, bag.nodePool.id);
    }
    catch (error) {
        logger.warn(`Cannot extract buildPlaneMessages count for the nodePoolId: ${bag.nodePool.id} with error:${error}`);
        throw error;
    }
}
function __findAvailableNodeCount(bag, next) {
    if (bag.error)
        return next();
    if (!bag.isValidLock)
        return next();
    if (_.isEmpty(bag.waitingGroups) &&
        _.isEmpty(bag.processingGroups))
        return next();
    if (bag.nodePool.isOrphaned)
        return next();
    var who = bag.who + '|' + __findAvailableNodeCount.name;
    logger.debug(who, 'Inside');
    const nodePoolRepository = new acl_1.NodePoolRepository();
    const availableNodeCountResponse = nodePoolRepository.findAvailableNodeCount(bag.nodePool.isOnDemand, bag.processingGroups, true, bag.buildPlaneMessagesCount, bag.nodePool.numberOfNodes, bag.statusCodes, bag.nodes, bag.nodePool.isCacheEnabled);
    bag.nodePool.totalGroupsThatCanBeQueued = availableNodeCountResponse.totalGroupsThatCanBeQueued;
    bag.nodePool.cachedNodes = availableNodeCountResponse.cachedNodes;
    bag.nodePool.activeNodesCount = availableNodeCountResponse.activeNodesCount;
    bag.nodePool.cachedNodesCount = availableNodeCountResponse.cachedNodesCount;
    return next();
}
async function __tryAndQueueStepsNew(bag) {
    if (bag.error)
        return;
    if (!bag.isValidLock)
        return;
    if (_.isEmpty(bag.waitingGroups))
        return;
    if (bag.nodePool.isOrphaned)
        return;
    var who = bag.who + '|' + __tryAndQueueStepsNew.name;
    logger.debug(who, 'Inside');
    const stepScheduler = new StepScheduler_1.StepScheduler(bag.waitingGroups, bag.nodePool.totalGroupsThatCanBeQueued, bag.nodePool, bag.groupsProjects);
    await stepScheduler.queueStepsToNodePool();
    return;
}
async function __releaseLock(bag) {
    if (!bag.isValidLock)
        return;
    var who = bag.who + '|' + __releaseLock.name;
    logger.debug(who, 'Inside');
    try {
        await bag.lockService.releaseLock();
        bag.lock = null;
        bag.isValidLock = false;
    }
    catch (error) {
        logger.warn('Failed to release lock due to error:', error);
    }
}
function _setResult(bag, next) {
    var who = bag.who + '|' + _setResult.name;
    logger.debug(who, 'Inside');
    bag.result = {
        waitingGroups: bag.waitingGroups,
        processingGroups: bag.processingGroups
    };
    return next();
}
function __publishToNPM(bag, next) {
    const who = bag.who + '|' + __publishToNPM.name;
    logger.debug(who, 'Inside');
    let msg = {
        where: 'core.nodePoolService',
        payload: {
            payload: {
                eventType: utility_1.Actor.NODE_POOL_PROCESSING,
                payload: {
                    id: bag.nodePool.id
                }
            }
        }
    };
    logger.trace('Sending the message to rabbitmq', JSON.stringify(msg));
    bag.suAdapter.postToVortex(msg, (err) => {
        if (err) {
            logger.debug(who, 'failed to :postToVortex for :nodePoolId', bag.nodePool.id, 'with error:', err);
        }
        return next();
    });
}
//# sourceMappingURL=queueStepsToNodePools.js.map