"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TryAndQueueExternalSteps = void 0;
const node_util_1 = require("node:util");
const logger_1 = require("@jfrog-pipelines/logger");
const utility_1 = require("@jfrog-pipelines/utility");
const internal_clients_1 = require("@jfrog-pipelines/internal-clients");
const acl_1 = require("../acl");
const stepConsoleAdapter_1 = __importDefault(require("../stepConsoleAdapter"));
const JOB_CONSOLE_BATCH_SIZE = 10;
const STEP_CONSOLE_BUFFER_TIME_INTERVAL = 3000;
class TryAndQueueExternalSteps {
    constructor(groups, sessionId, apiToken) {
        this.groups = groups;
        this.logger = logger_1.LoggerFactory.getLogger('TryAndQueueExternalSteps');
        this.waitingGroups = groups;
        this.nodePoolAcl = new acl_1.NodePoolRepository();
        this.sessionId = sessionId;
        this.apiToken = apiToken;
        this.hookIdsByStepId = {};
        this.client = new internal_clients_1.PipelinesClient(12, this.apiToken);
    }
    async queueSteps() {
        const tryAndQueueExternalStepsWorkerPromises = this.waitingGroups.map((group) => {
            return this.tryAndQueueExternalStepsWorker(group);
        });
        await Promise.all(tryAndQueueExternalStepsWorkerPromises);
    }
    async tryAndQueueExternalStepsWorker(group) {
        try {
            await this.checkLicenseCapacity();
            await this.updateExternalStep(group);
            await this.queueExternalStep(group);
            this.postExternalStepConsoles(group);
        }
        catch (error) {
            this.logger.warn(error);
        }
    }
    async checkLicenseCapacity() {
        let rejectAll = false;
        let executionNodes = {};
        try {
            executionNodes = await this.nodePoolAcl.getExecutionNodes();
            if (executionNodes.available <= 0) {
                rejectAll = true;
            }
        }
        catch (error) {
            this.logger.warn('Cannot extract execution Nodes. Got error:', (0, node_util_1.inspect)(error));
            rejectAll = true;
        }
        if (!rejectAll) {
            return;
        }
        this.processLicenseCapacity(this.waitingGroups);
    }
    processLicenseCapacity(waitingGroups) {
        for (const group of waitingGroups) {
            group.reject = true;
        }
    }
    async updateExternalStep(group) {
        if (group.reject) {
            this.logger.warn('Group Rejected');
            return;
        }
        const update = {
            groupInProgress: true,
            groupStartedAt: new Date()
        };
        group.groupStartedAt = update.groupStartedAt;
        const batchSize = 10;
        try {
            await (0, utility_1.eachLimit)(group.groupStepIds, batchSize, async (stepObject) => {
                try {
                    const step = await this.updateStep(stepObject.stepId, update);
                    if (step.configPropertyBag.hookId) {
                        const configPropertyBag = step.configPropertyBag;
                        if (configPropertyBag.hookId) {
                            this.hookIdsByStepId[step.id] = configPropertyBag.hookId;
                        }
                    }
                }
                catch (error) {
                    this.logger.warn(`Cannot update step with stepId:${stepObject.stepId} with error: ${(0, node_util_1.inspect)(error)}`);
                }
            });
        }
        catch (error) {
            this.logger.warn(error);
            throw error;
        }
    }
    async queueExternalStep(group) {
        if (group.reject) {
            return;
        }
        const stepId = (0, utility_1.map)(group.groupStepIds, 'stepId')[0];
        const messagePayload = this.generateMessagePayload(stepId);
        try {
            const hookId = this.hookIdsByStepId[stepId];
            await this.client.postHookWebHook(hookId, messagePayload);
        }
        catch (error) {
            this.logger.warn(`Cannot post webhook for the stepId:${stepId} and webhookId: ${this.hookIdsByStepId[stepId]} with error:${(0, node_util_1.inspect)(error)}`);
        }
    }
    postExternalStepConsoles(group) {
        const groupStepIds = group.groupStepIds;
        for (const stepObject of groupStepIds) {
            const consoleAdapter = new stepConsoleAdapter_1.default(this.sessionId, this.apiToken, stepObject.stepId, group.pipelineId, JOB_CONSOLE_BATCH_SIZE, STEP_CONSOLE_BUFFER_TIME_INTERVAL);
            const logMessage = `Successfully queued step. Waiting for execution to start`;
            consoleAdapter.openGrp('Queuing Step');
            consoleAdapter.openCmd('Info');
            consoleAdapter.publishMsg(logMessage);
            consoleAdapter.closeCmd(true);
            consoleAdapter.closeGrp(true);
        }
    }
    generateMessagePayload(stepId) {
        return {
            action: 'trigger',
            stepId: stepId
        };
    }
    async updateStep(stepId, update) {
        try {
            const step = await this.client.putStepById(stepId, update);
            return step;
        }
        catch (error) {
            this.logger.warn(`Cannot update step, facing error: ${(0, node_util_1.inspect)(error)}`);
            throw error;
        }
    }
}
exports.TryAndQueueExternalSteps = TryAndQueueExternalSteps;
//# sourceMappingURL=tryAndQueueExternalStepsNew.js.map