"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DagCollectionFilter = void 0;
const metadata_1 = require("@jfrog-pipelines/metadata");
const utility_1 = require("@jfrog-pipelines/utility");
const data_access_layer_1 = require("@jfrog-pipelines/data-access-layer");
const RunDao_1 = require("../../driven/db/RunDao");
const StepDao_1 = require("../../driven/db/StepDao");
const PipelineDao_1 = require("../../driven/db/PipelineDao");
class DagCollectionFilter {
    static async filter(dagRecords) {
        const filteredListAfterDependencyMode = await this.filterListByDependencyMode(dagRecords);
        const filteredListAfterChronologicalFlag = await this.filterByChronologicalFlag(filteredListAfterDependencyMode);
        return filteredListAfterChronologicalFlag;
    }
    static async filterListByDependencyMode(dagRecords) {
        const parentCompletePipelineIdsSet = new Set();
        const childCompletePipelineIdsSet = new Set();
        const parentSuccessPipelineIdsSet = new Set();
        for (const record of dagRecords) {
            this.populateDependencyPipelines(record, parentCompletePipelineIdsSet, parentSuccessPipelineIdsSet, childCompletePipelineIdsSet);
        }
        const allPipelineIds = new Set([
            ...parentCompletePipelineIdsSet.values(),
            ...childCompletePipelineIdsSet.values(),
            ...parentSuccessPipelineIdsSet.values()
        ]);
        if (allPipelineIds.size === 0) {
            return dagRecords;
        }
        const incompleteStatusCodes = [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.TIMINGOUT, metadata_1.CANCELLING];
        const pipelineIds = [...allPipelineIds.values()];
        const runDao = new RunDao_1.RunDao();
        const incompleteRuns = await runDao.fetchIncompleteRuns(pipelineIds, incompleteStatusCodes);
        const incompleteRunsByPipelineId = (0, utility_1.groupBy)(incompleteRuns, 'pipelineId');
        return this.checkAndFilterRunAdjacencies(allPipelineIds, incompleteRunsByPipelineId, parentSuccessPipelineIdsSet, dagRecords);
    }
    static async filterByChronologicalFlag(dagRecords) {
        const filteredListAfterChronologicalRun = await this.filterByChronologicalRun(dagRecords);
        const filteredListAfterChronologicalStep = await this.filterByChronologicalStep(filteredListAfterChronologicalRun);
        return filteredListAfterChronologicalStep;
    }
    static populateDependencyPipelines(runAdj, parentCompletePipelineIdsSet, parentSuccessPipelineIdsSet, childCompletePipelineIdsSet) {
        if (runAdj.dependentEntityType === 3) {
            parentCompletePipelineIdsSet.add(runAdj.dependentEntityId);
        }
        if (runAdj.dependentEntityType === 4) {
            parentSuccessPipelineIdsSet.add(runAdj.dependentEntityId);
        }
        if (runAdj.dependentEntityType === 5) {
            childCompletePipelineIdsSet.add(runAdj.dependentEntityId);
        }
    }
    static async filterByChronologicalRun(runAdjacencyList) {
        const chronologicalPipelineIdsSet = new Set();
        for (const record of runAdjacencyList) {
            if (record.isChronologicalRun) {
                chronologicalPipelineIdsSet.add(record.pipelineId);
            }
        }
        if (chronologicalPipelineIdsSet.size === 0) {
            return runAdjacencyList;
        }
        const chronologicalRunPipelineIds = [...chronologicalPipelineIdsSet];
        const chronologicalRunsByPipelineIds = await this.getIncompleteRunsByPipelineIds(chronologicalRunPipelineIds);
        const filteredList = this.getFilteredListForChronologicalRun(chronologicalRunsByPipelineIds, runAdjacencyList);
        return filteredList;
    }
    static async getIncompleteRunsByPipelineIds(pipelineIds) {
        const runDao = new RunDao_1.RunDao();
        const bulder = new data_access_layer_1.SequelizeQueryBuilder({});
        const query = bulder
            .where('statusCode', [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.TIMINGOUT, metadata_1.CANCELLING])
            .where('pipelineId', pipelineIds)
            .attributes(['id', 'pipelineId', 'runNumber'])
            .sort()
            .build();
        const incompleteRuns = await runDao.findAll(query);
        const incompleteRunsByPipelineId = (0, utility_1.groupBy)(incompleteRuns, 'pipelineId');
        return incompleteRunsByPipelineId;
    }
    static getFilteredListForChronologicalRun(chronologicalRunsByPipelineIds, runAdjacencyList) {
        const rejectedRunIds = new Set();
        for (const id in chronologicalRunsByPipelineIds) {
            const pipelineId = Number.parseInt(id, 10);
            const runsSortedByRunNumber = (0, utility_1.sortBy)(chronologicalRunsByPipelineIds[pipelineId], 'runNumber');
            for (let index = 1; index < runsSortedByRunNumber.length; index++) {
                rejectedRunIds.add(Number(runsSortedByRunNumber[index].id));
            }
        }
        const filteredList = runAdjacencyList.filter((record) => {
            if (rejectedRunIds.has(record.runId)) {
                return false;
            }
            return true;
        });
        return filteredList;
    }
    static async filterByChronologicalStep(runAdjacencyList) {
        const chronologicalStepIdsSet = new Set();
        for (const record of runAdjacencyList) {
            if (record.isChronologicalStep) {
                chronologicalStepIdsSet.add(record.chronologicalPipelineStepId);
            }
        }
        if (chronologicalStepIdsSet.size === 0) {
            return runAdjacencyList;
        }
        const chronologicalPipelineStepIds = [...chronologicalStepIdsSet];
        const chronologicalStepsByPipelineStepIds = await this.getIncompleteStepsByStepIds(chronologicalPipelineStepIds);
        const [rejectableAffinityGroups, rejectableRunIds, rejectableStepIds] = this.computeRejectableEntitiesForChronologicalStep(chronologicalStepsByPipelineStepIds);
        return runAdjacencyList.filter((record) => {
            if (this.getRejectableConditionForChronologicalStep(rejectableRunIds, record, rejectableStepIds, rejectableAffinityGroups)) {
                return false;
            }
            return true;
        });
    }
    static getRejectableConditionForChronologicalStep(rejectableRunIds, record, rejectableStepIds, rejectableAffinityGroups) {
        return (rejectableRunIds.has(record.runId) &&
            (rejectableStepIds.has(record.stepId) || rejectableAffinityGroups.has(record.affinityGroup)));
    }
    static computeRejectableEntitiesForChronologicalStep(chronologicalStepsByPipelineStepIds) {
        const rejectableAffinityGroups = new Set();
        const rejectableRunIds = new Set();
        const rejectableStepIds = new Set();
        for (const id in chronologicalStepsByPipelineStepIds) {
            const pipelineStepId = Number.parseInt(id, 10);
            const stepsSortedByRunId = (0, utility_1.sortBy)(chronologicalStepsByPipelineStepIds[pipelineStepId], 'runId');
            for (let index = 1; index < stepsSortedByRunId.length; index++) {
                const step = stepsSortedByRunId[index];
                rejectableStepIds.add(Number(step.id));
                rejectableAffinityGroups.add(step.affinityGroup);
                rejectableRunIds.add(Number(step.runId));
            }
        }
        return [rejectableAffinityGroups, rejectableRunIds, rejectableStepIds];
    }
    static async getIncompleteStepsByStepIds(pipelineStepIds) {
        const stepDao = new StepDao_1.StepDao();
        const bulder = new data_access_layer_1.SequelizeQueryBuilder({});
        const query = bulder
            .where('statusCode', [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.TIMINGOUT, metadata_1.CANCELLING])
            .where('pipelineStepId', pipelineStepIds)
            .attributes(['id', 'pipelineId', 'pipelineStepId', 'affinityGroup', 'runId'])
            .sort()
            .build();
        const incompleteSteps = await stepDao.findAll(query);
        const incompleteStepsByPipelineStepId = (0, utility_1.groupBy)(incompleteSteps, 'pipelineStepId');
        return incompleteStepsByPipelineStepId;
    }
    static checkAndFilterRunAdjacencies(allPipelineIds, incompleteRunsByPipelineId, parentSuccessPipelineIdsSet, runAdjacencyList) {
        const checkForParentSuccess = new Set();
        const dependencyModePipelinesUnresolved = new Set();
        for (const pipelineId of allPipelineIds) {
            const incompleteRuns = incompleteRunsByPipelineId[pipelineId];
            if (!(0, utility_1.isEmpty)(incompleteRuns)) {
                dependencyModePipelinesUnresolved.add(pipelineId);
            }
            else {
                if (parentSuccessPipelineIdsSet.has(pipelineId)) {
                    checkForParentSuccess.add(pipelineId);
                }
            }
        }
        return this.removeRunAdjacencies(checkForParentSuccess, dependencyModePipelinesUnresolved, runAdjacencyList);
    }
    static async removeRunAdjacencies(checkForParentSuccess, dependencyModePipelinesUnresolved, runAdjacencyList) {
        if (checkForParentSuccess.size > 0) {
            const pipelineIds = [...checkForParentSuccess.values()];
            await this.checkForParentSuccess(pipelineIds, dependencyModePipelinesUnresolved);
        }
        if (dependencyModePipelinesUnresolved.size === 0) {
            return runAdjacencyList;
        }
        const unresolvedPipelines = this.getPipelinesWaitingForDependencyMode(dependencyModePipelinesUnresolved, runAdjacencyList);
        return this.filterUnresolvedDependencyModePipelines(unresolvedPipelines, runAdjacencyList);
    }
    static async checkForParentSuccess(pipelineIds, dependencyModePipelinesUnresolved) {
        const pipelineDao = new PipelineDao_1.PipelineDao();
        const query = new data_access_layer_1.SequelizeQueryBuilder({}).where('id', pipelineIds).build();
        const pipelines = await pipelineDao.findAll(query);
        const pipelinesById = (0, utility_1.keyBy)(pipelines, 'id');
        for (const pipelineId of pipelineIds) {
            const pipeline = pipelinesById[pipelineId];
            if (!(0, utility_1.isEmpty)(pipeline) && pipeline.latestCompletedRunId !== pipeline.latestSuccessfulRunId) {
                dependencyModePipelinesUnresolved.add(pipelineId);
            }
        }
    }
    static getPipelinesWaitingForDependencyMode(pipelineIdSet, runAdjacencyList) {
        const dependencyModeEntityTypes = new Set([
            5,
            3,
            4
        ]);
        const waitingPipelines = new Set();
        for (const runAdj of runAdjacencyList) {
            const dependentEntityType = runAdj.dependentEntityType;
            const dependentEntityId = runAdj.dependentEntityId;
            if (!dependencyModeEntityTypes.has(dependentEntityType)) {
                continue;
            }
            if (pipelineIdSet.has(dependentEntityId)) {
                waitingPipelines.add(runAdj.pipelineId);
            }
        }
        return waitingPipelines;
    }
    static filterUnresolvedDependencyModePipelines(unresolvedPipelines, runAdjacencyList) {
        return runAdjacencyList.filter((runAdj) => {
            return !unresolvedPipelines.has(runAdj.pipelineId);
        });
    }
}
exports.DagCollectionFilter = DagCollectionFilter;
//# sourceMappingURL=DagCollectionFilter.js.map